﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  [Item("PermutationUnwantedMutationAnalyzer", "An item that calculates unwanted mutations for the permutation encoding.")]
  [StorableClass]
  public class PermutationUnwantedMutationAnalyzer : UnwantedMutationAnalyzer {
    [StorableConstructor]
    protected PermutationUnwantedMutationAnalyzer(bool deserializing) : base(deserializing) { }
    protected PermutationUnwantedMutationAnalyzer(PermutationUnwantedMutationAnalyzer original, Cloner cloner)
      : base(original, cloner) { }

    public PermutationUnwantedMutationAnalyzer()
      : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PermutationUnwantedMutationAnalyzer(this, cloner);
    }

    public override double AnalyzeUnwantedMutations(IScope parent1, IScope parent2, IScope child) {
      Permutation p1 = parent1.Variables[SolutionVariableName].Value as Permutation;
      Permutation p2 = parent2.Variables[SolutionVariableName].Value as Permutation;
      Permutation c = child.Variables[SolutionVariableName].Value as Permutation;

      return AnalyzeUnwantedMutations(p1, p2, c);
    }

    private double AnalyzeUnwantedMutations(Permutation parent1, Permutation parent2, Permutation child) {
      int cnt = 0;
      int[] edgesP1 = CalculateEdgesVector(parent1);
      int[] edgesP2 = CalculateEdgesVector(parent2);
      int[] edgesC = CalculateEdgesVector(child);

      for (int i = 0; i < edgesP1.Length; i++) {
        if (edgesC[i] != edgesP1[i] &&
            edgesC[i] != edgesP2[i] &&
            edgesP1[edgesC[i]] != i &&
            edgesP2[edgesC[i]] != i) {
          cnt += 1;
        }
      }

      //reverse so that it matches the other diversity charts
      return 1.0 - (((double)cnt) / parent1.Length);
    }

    //copied from PermutationEqualityComparer
    private int[] CalculateEdgesVector(Permutation permutation) {
      // transform path representation into adjacency representation
      int[] edgesVector = new int[permutation.Length];
      for (int i = 0; i < permutation.Length - 1; i++)
        edgesVector[permutation[i]] = permutation[i + 1];
      edgesVector[permutation[permutation.Length - 1]] = permutation[0];
      return edgesVector;
    }
  }
}
